#ifndef EMBEDDINGENGINE_H
#define EMBEDDINGENGINE_H

#include <functional>
#include <string>
#include <vector>

#include "aiengine.h"

using EmbeddingResultCallback = std::function<void(std::vector<float> result)>;

namespace ai_engine {
namespace embedding {

class AbstractTextEmbeddingEngine : public ai_engine::AbstractAiEngine {
public:
    virtual std::string engineName() const = 0;

    virtual void embeddingTextAsync(const std::string& input,
                                    EmbeddingResultCallback callback) = 0;
};

class AbstractImageEmbeddingEngine : public ai_engine::AbstractAiEngine {
public:
    virtual std::string engineName() const = 0;

    virtual void embeddingTextAsync(const std::string& input,
                                    EmbeddingResultCallback callback) = 0;
    virtual void embeddingImageFromImageFileAsync(
        const std::string& file_path, EmbeddingResultCallback callback) = 0;
    virtual void embeddingImageFromBase64ImageDataAsync(
        const std::string& file_data, EmbeddingResultCallback callback) = 0;
};

}  // namespace embedding
}  // namespace ai_engine

#endif  // EMBEDDINGENGINE_H
