/*
 * Copyright 2024 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef VISIONENGINE_H
#define VISIONENGINE_H

#include <functional>
#include <string>
#include <vector>

#include "aiengine.h"
#include "aiengineerror.h"

namespace ai_engine {
namespace vision {

struct ImageData {
    int format;
    int height;
    int width;
    int total;  // 总共生成的图片数量
    int index;  // 当前数据对应的图片的索引
    std::vector<char> imageData;
    EngineError error;
};

typedef struct 
{
    int x;
    int y;
} PixelPoint;

struct TextLine
{
    std::string text;
    std::vector<PixelPoint> points;
    float conf;
};

struct TextRecognitionResult {
    std::string allText;
    std::vector<TextLine> lines;
    int errorCode;
    std::string errorMessage;
};
using ImageResultCallback = std::function<void(ImageData imageData)>;
using OcrResultCallback = std::function<void(TextRecognitionResult result)>;

class AbstractVisionEngine : public ai_engine::AbstractAiEngine {
public:
    virtual ~AbstractVisionEngine() {}

    virtual int maxConcurrentPrompt2ImageTasks() const {return 0;};

    virtual void setPrompt2ImageCallback(ImageResultCallback callback) {};

    virtual void setOcrCallback(OcrResultCallback callback) {};
    virtual void setPrompt2ImageSize(int width, int height) {}
    virtual void setPrompt2ImageNumber(int number) {}
    virtual void setPrompt2ImageStyle(int style) {}
    virtual void setPrompt2ImageStyle(const std::string &style) {}
    virtual std::string supportedPrompt2ImageParams() const {return 0;};

    virtual bool prompt2Image(const std::string &prompt,
                              ai_engine::EngineError &error) {return false;};

    virtual bool ocrRequestPath(std::string ImagePath, EngineError &error) {return false;};

    virtual bool ocrRequestBase64(const char *ImageData, EngineError &error) {return false;};

    virtual bool initOcrModule(EngineError &error) {return false;};
    
    virtual bool destroyOcrModule(EngineError &error) {return false;};
};

}  // namespace vision
}  // namespace ai_engine

#endif  // VISIONENGINE_H
